# Tests for the test_new Metropolis SystemC simulator using metroshell
#
# @Author: Christopher Hylands
#
# @Version: $Id: metroshell.tcl,v 1.9 2004/05/20 03:18:08 cxh Exp $
#
# @Copyright (c) 2003-2004 The Regents of the University of California.
# All rights reserved.
#
# Permission is hereby granted, without written agreement and without
# license or royalty fees, to use, copy, modify, and distribute this
# software and its documentation for any purpose, provided that the
# above copyright notice and the following two paragraphs appear in all
# copies of this software and that appropriate acknowledgments are made
# to the research of the Metropolis group.
# 
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
# THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
# PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
# CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.
#
# 						PT_COPYRIGHT_VERSION_2
# 						COPYRIGHTENDKEY
#######################################################################

# Load up the test definitions.
if {[string compare test [info procs test]] == 1} then {
    source testDefs.tcl
} {}

# Uncomment this to get a full report, or set in your Tcl shell window.
# set VERBOSE 1

######################################################################
####
#
test metroshell-1.1 {Initial setup} {
    file delete -force [glob -nocomplain *.tmp *.bin *.o *.cpp]
    file delete -force systemc_sim.mk run.x 

    classpath add ../..
    metroload pkg -semantics producers_consumer
    elaborate producers_consumer.IwIr
    simulate systemc producers_consumer.IwIr
    list \
	    [file exists sc_main.cpp ] \
	    [file exists systemc_sim.mk]
} {1 1}

test metroshell-1.2 {Run make} {
    puts "Running 'make -f systemc_sim.mk' - this could take awhile"
    exec make -f systemc_sim.mk
    puts "... Done running make"
    # file executable run.x does not work under Windows?
    file exists run.x
} {1}

test metroshell-1.3 {Run the binary} {
    set results [exec -stderrok ".[java::field java.io.File separator]run.x"]
    regsub {^ *SystemC 2\.0.*$} $results {} results2
    regsub {^ *Copyright.*$} $results2 {} results3
    # Substitute Producer? for Producer0 and Producer1
    regsub -all {Producer0} $results3 {Producer?} results4
    regsub -all {Producer1} $results4 {Producer?} results5
    list $results5
} {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
monitor> Producer?: write 0 BEGIN_RT
monitor> Producer?: write 10 END_RT
monitor> Consumer: read 10 BEGIN_RT
monitor> Consumer: read 20 END_RT
monitor> Producer?: write 20 BEGIN_RT
monitor> Producer?: write 30 END_RT
monitor> Consumer: read 30 BEGIN_RT
monitor> Consumer: read 40 END_RT
monitor> Producer?: write 40 BEGIN_RT
monitor> Producer?: write 50 END_RT
monitor> Consumer: read 50 BEGIN_RT
monitor> Consumer: read 60 END_RT
monitor> Producer?: write 60 BEGIN_RT
monitor> Producer?: write 70 END_RT
monitor> Consumer: read 70 BEGIN_RT
monitor> Consumer: read 80 END_RT
monitor> Producer?: write 80 BEGIN_RT
monitor> Producer?: write 90 END_RT
monitor> Consumer: read 90 BEGIN_RT
monitor> Consumer: read 100 END_RT
monitor> Producer?: write 100 BEGIN_RT
monitor> Producer?: write 110 END_RT
monitor> Consumer: read 110 BEGIN_RT
monitor> Consumer: read 120 END_RT
monitor> Producer?: write 120 BEGIN_RT
monitor> Producer?: write 130 END_RT
monitor> Consumer: read 130 BEGIN_RT
monitor> Consumer: read 140 END_RT
monitor> Producer?: write 140 BEGIN_RT
monitor> Producer?: write 150 END_RT
monitor> Consumer: read 150 BEGIN_RT
monitor> Consumer: read 160 END_RT
monitor> Producer?: write 160 BEGIN_RT
monitor> Producer?: write 170 END_RT
monitor> Consumer: read 170 BEGIN_RT
monitor> Consumer: read 180 END_RT
monitor> Producer?: write 180 BEGIN_RT
monitor> Producer?: write 190 END_RT
monitor> Consumer: read 190 BEGIN_RT
monitor> Consumer: read 200 END_RT
monitor> Producer?: write 200 BEGIN_RT
monitor> Producer?: write 210 END_RT
monitor> Consumer: read 210 BEGIN_RT
monitor> Consumer: read 220 END_RT
monitor> Producer?: write 220 BEGIN_RT
monitor> Producer?: write 230 END_RT
monitor> Consumer: read 230 BEGIN_RT
monitor> Consumer: read 240 END_RT
monitor> Producer?: write 240 BEGIN_RT
monitor> Producer?: write 250 END_RT
monitor> Consumer: read 250 BEGIN_RT
monitor> Consumer: read 260 END_RT
monitor> Producer?: write 260 BEGIN_RT
monitor> Producer?: write 270 END_RT
monitor> Consumer: read 270 BEGIN_RT
monitor> Consumer: read 280 END_RT
monitor> Producer?: write 280 BEGIN_RT
monitor> Producer?: write 290 END_RT
monitor> Consumer: read 290 BEGIN_RT
monitor> Consumer: read 300 END_RT
monitor> Producer?: write 300 BEGIN_RT
monitor> Producer?: write 310 END_RT
monitor> Consumer: read 310 BEGIN_RT
monitor> Consumer: read 320 END_RT
monitor> Producer?: write 320 BEGIN_RT
monitor> Producer?: write 330 END_RT
monitor> Consumer: read 330 BEGIN_RT
monitor> Consumer: read 340 END_RT
monitor> Producer?: write 340 BEGIN_RT
monitor> Producer?: write 350 END_RT
monitor> Consumer: read 350 BEGIN_RT
monitor> Consumer: read 360 END_RT
monitor> Producer?: write 360 BEGIN_RT
monitor> Producer?: write 370 END_RT
monitor> Consumer: read 370 BEGIN_RT
monitor> Consumer: read 380 END_RT
monitor> Producer?: write 380 BEGIN_RT
monitor> Producer?: write 390 END_RT
monitor> Consumer: read 390 BEGIN_RT
monitor> Consumer: read 400 END_RT
monitor> Producer?: write 400 BEGIN_RT
monitor> Producer?: write 410 END_RT
monitor> Consumer: read 410 BEGIN_RT
monitor> Consumer: read 420 END_RT
monitor> Producer?: write 420 BEGIN_RT
monitor> Producer?: write 430 END_RT
monitor> Consumer: read 430 BEGIN_RT
monitor> Consumer: read 440 END_RT
monitor> Producer?: write 440 BEGIN_RT
monitor> Producer?: write 450 END_RT
monitor> Consumer: read 450 BEGIN_RT
monitor> Consumer: read 460 END_RT
monitor> Producer?: write 460 BEGIN_RT
monitor> Producer?: write 470 END_RT
monitor> Consumer: read 470 BEGIN_RT
monitor> Consumer: read 480 END_RT
monitor> Producer?: write 480 BEGIN_RT
monitor> Producer?: write 490 END_RT
monitor> Consumer: read 490 BEGIN_RT
monitor> Consumer: read 500 END_RT
monitor> Producer?: write 500 BEGIN_RT
monitor> Producer?: write 510 END_RT
monitor> Consumer: read 510 BEGIN_RT
monitor> Consumer: read 520 END_RT
monitor> Producer?: write 520 BEGIN_RT
monitor> Producer?: write 530 END_RT
monitor> Consumer: read 530 BEGIN_RT
monitor> Consumer: read 540 END_RT
monitor> Producer?: write 540 BEGIN_RT
monitor> Producer?: write 550 END_RT
monitor> Consumer: read 550 BEGIN_RT
monitor> Consumer: read 560 END_RT
monitor> Producer?: write 560 BEGIN_RT
monitor> Producer?: write 570 END_RT
monitor> Consumer: read 570 BEGIN_RT
monitor> Consumer: read 580 END_RT
monitor> Producer?: write 580 BEGIN_RT
monitor> Producer?: write 590 END_RT
monitor> Consumer: read 590 BEGIN_RT
monitor> Consumer: read 600 END_RT
monitor> Producer?: write 600 BEGIN_RT
monitor> Producer?: write 610 END_RT
monitor> Consumer: read 610 BEGIN_RT
monitor> Consumer: read 620 END_RT
monitor> Producer?: write 620 BEGIN_RT
monitor> Producer?: write 630 END_RT
monitor> Consumer: read 630 BEGIN_RT
monitor> Consumer: read 640 END_RT
monitor> Producer?: write 640 BEGIN_RT
monitor> Producer?: write 650 END_RT
monitor> Consumer: read 650 BEGIN_RT
monitor> Consumer: read 660 END_RT
monitor> Producer?: write 660 BEGIN_RT
monitor> Producer?: write 670 END_RT
monitor> Consumer: read 670 BEGIN_RT
monitor> Consumer: read 680 END_RT
monitor> Producer?: write 680 BEGIN_RT
monitor> Producer?: write 690 END_RT
monitor> Consumer: read 690 BEGIN_RT
monitor> Consumer: read 700 END_RT
monitor> Producer?: write 700 BEGIN_RT
monitor> Producer?: write 710 END_RT
monitor> Consumer: read 710 BEGIN_RT
monitor> Consumer: read 720 END_RT
monitor> Producer?: write 720 BEGIN_RT
monitor> Producer?: write 730 END_RT
monitor> Consumer: read 730 BEGIN_RT
monitor> Consumer: read 740 END_RT
monitor> Producer?: write 740 BEGIN_RT
monitor> Producer?: write 750 END_RT
monitor> Consumer: read 750 BEGIN_RT
monitor> Consumer: read 760 END_RT
monitor> Producer?: write 760 BEGIN_RT
monitor> Producer?: write 770 END_RT
monitor> Consumer: read 770 BEGIN_RT
monitor> Consumer: read 780 END_RT
monitor> Producer?: write 780 BEGIN_RT
monitor> Producer?: write 790 END_RT
monitor> Consumer: read 790 BEGIN_RT
monitor> Consumer: read 800 END_RT
monitor> Producer?: write 800 BEGIN_RT
monitor> Producer?: write 810 END_RT
monitor> Consumer: read 810 BEGIN_RT
monitor> Consumer: read 820 END_RT
monitor> Producer?: write 820 BEGIN_RT
monitor> Producer?: write 830 END_RT
monitor> Consumer: read 830 BEGIN_RT
monitor> Consumer: read 840 END_RT
monitor> Producer?: write 840 BEGIN_RT
monitor> Producer?: write 850 END_RT
monitor> Consumer: read 850 BEGIN_RT
monitor> Consumer: read 860 END_RT
monitor> Producer?: write 860 BEGIN_RT
monitor> Producer?: write 870 END_RT
monitor> Consumer: read 870 BEGIN_RT
monitor> Consumer: read 880 END_RT
monitor> Producer?: write 880 BEGIN_RT
monitor> Producer?: write 890 END_RT
monitor> Consumer: read 890 BEGIN_RT
monitor> Consumer: read 900 END_RT
monitor> Producer?: write 900 BEGIN_RT
monitor> Producer?: write 910 END_RT
monitor> Consumer: read 910 BEGIN_RT
monitor> Consumer: read 920 END_RT
monitor> Producer?: write 920 BEGIN_RT
monitor> Producer?: write 930 END_RT
monitor> Consumer: read 930 BEGIN_RT
monitor> Consumer: read 940 END_RT
monitor> Producer?: write 940 BEGIN_RT
monitor> Producer?: write 950 END_RT
monitor> Consumer: read 950 BEGIN_RT
monitor> Consumer: read 960 END_RT
monitor> Producer?: write 960 BEGIN_RT
monitor> Producer?: write 970 END_RT
monitor> Consumer: read 970 BEGIN_RT
monitor> Consumer: read 980 END_RT
monitor> Producer?: write 980 BEGIN_RT
monitor> Producer?: write 990 END_RT
monitor> Consumer: read 990 BEGIN_RT
monitor> Consumer: read 1000 END_RT
monitor> Producer?: write 1000 BEGIN_RT
monitor> Producer?: write 1010 END_RT
monitor> Consumer: read 1010 BEGIN_RT
monitor> Consumer: read 1020 END_RT
monitor> Producer?: write 1020 BEGIN_RT
monitor> Producer?: write 1030 END_RT
monitor> Consumer: read 1030 BEGIN_RT
monitor> Consumer: read 1040 END_RT
monitor> Producer?: write 1040 BEGIN_RT
monitor> Producer?: write 1050 END_RT
monitor> Consumer: read 1050 BEGIN_RT
monitor> Consumer: read 1060 END_RT
monitor> Producer?: write 1060 BEGIN_RT
monitor> Producer?: write 1070 END_RT
monitor> Consumer: read 1070 BEGIN_RT
monitor> Consumer: read 1080 END_RT
monitor> Producer?: write 1080 BEGIN_RT
monitor> Producer?: write 1090 END_RT
monitor> Consumer: read 1090 BEGIN_RT
monitor> Consumer: read 1100 END_RT
monitor> Producer?: write 1100 BEGIN_RT
monitor> Producer?: write 1110 END_RT
monitor> Consumer: read 1110 BEGIN_RT
monitor> Consumer: read 1120 END_RT
SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}

test metroshell-2.1 {network show} {
    # Uses setup from metroshell-1.1 above
    set results [network show]
    # Unfortunately, the order of Components varies
    list \
	    [list \
	    [regexp {Top-level netlist:} $results] \
	    [regexp {netlist producers_consumer.IwIr} $results] \
	    [regexp {o Instance name: top_level_netlist} $results] \
	    [regexp {o Component name:} $results] \
	    [regexp {  o Components:} $results]] \
	    [list \
	    [regexp {MEDIUM  \(instance name: InstIntM\)} $results] \
	    [regexp {DummyWriter  \(instance name: DummyW\)} $results] \
	    [regexp {DummyReader  \(instance name: DummyR\)} $results] \
	    [regexp {Producer1  \(instance name: Producer1\)} $results] \
	    [regexp {Producer0  \(instance name: Producer0\)} $results] \
	    [regexp {Consumer  \(instance name: Consumer\)} $results] \
	    ] \
	    [list \
	    [regexp {o Not refined by a netlist} $results] \
	    [regexp {o Does not refine any node} $results] \
	    [regexp {o No constraints} $results]]

} {{1 1 1 1 1} {1 1 1 1 1 1} {1 1 1}}
