# Tests for the strongarm
#
# @Author: Christopher Hylands Brooks
#
# @Version: $Id: strongarm_yapi.tcl,v 1.1 2004/06/04 21:35:34 cxh Exp $
#
# @Copyright (c) 2004 The Regents of the University of California.
# All rights reserved.
#
# Permission is hereby granted, without written agreement and without
# license or royalty fees, to use, copy, modify, and distribute this
# software and its documentation for any purpose, provided that the
# above copyright notice and the following two paragraphs appear in all
# copies of this software and that appropriate acknowledgments are made
# to the research of the Metropolis group.
# 
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
# THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
# PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
# CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.
#
# 						PT_COPYRIGHT_VERSION_2
# 						COPYRIGHTENDKEY
#######################################################################

# Load up the test definitions.
if {[string compare test [info procs test]] == 1} then {
    source testDefs.tcl
} {}

# Uncomment this to get a full report, or set in your Tcl shell window.
# set VERBOSE 1

# Build ../strongarm.x
set TESTDIR [pwd]

cd ..
if [catch {exec -stderrok make all} errMsg] {
    cd $TESTDIR
    error "ProducersConsumer-1.1 failed:  $errMsg"
}
cd $TESTDIR

if {! [ file exists ..[java::field java.io.File separator]strongarm.x]} {
    error "[pwd]..[java::field java.io.File separator]strongarm.x does not exist"
}

######################################################################
####
#
# Run strongarm.x on a tracefile, expect a known good results
#
proc StrongArmTest {testNumber tracefile results} {
    test Strongarm-$testNumber "run $tracefile" {
	set results1 [exec -stderrok \
			 ../strongarm.x $tracefile]
	#puts $results1
	regsub {^ *SystemC 2\.0.*$} $results1 {} results2
	regsub {^ *Copyright.*$} $results2 {} results3
	list $results3
    } $results
}


######################################################################
####
#
StrongArmTest 1.1 ../../object_files/arith2.tr \
    {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
TRACE FILE IS:../../object_files/arith2.tr
Mult:1
Mult:1
Fetch--- Cycles: 1179 Stalls:541
Fetch--- Branches: 96 Mispredicts:0

Fetch--- Cycles: 1180 Stalls:541
Fetch--- Branches: 96 Mispredicts:0

Fetch--- Cycles: 1181 Stalls:541
Fetch--- Branches: 96 Mispredicts:0


Cycles: 1182  Stalls: 52  Bubbles: 491
Branches: 96  Mispredicts: 0  BTB_Loads: 0

SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}

######################################################################
####
#
StrongArmTest 1.2 ../../object_files/arith-core.tr \
    {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
TRACE FILE IS:../../object_files/arith-core.tr
Fetch--- Cycles: 9 Stalls:2
Fetch--- Branches: 0 Mispredicts:0

Fetch--- Cycles: 10 Stalls:2
Fetch--- Branches: 0 Mispredicts:0

Mult:1
Fetch--- Cycles: 11 Stalls:2
Fetch--- Branches: 0 Mispredicts:0

Fetch--- Cycles: 12 Stalls:2
Fetch--- Branches: 0 Mispredicts:0


Cycles: 13  Stalls: 1  Bubbles: 4
Branches: 0  Mispredicts: 0  BTB_Loads: 0

SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}

######################################################################
####
#
StrongArmTest 1.3 ../../object_files/arith.tr \
    {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
TRACE FILE IS:../../object_files/arith.tr
Mult:1
Fetch--- Cycles: 1159 Stalls:530
Fetch--- Branches: 96 Mispredicts:0

Fetch--- Cycles: 1160 Stalls:530
Fetch--- Branches: 96 Mispredicts:0

Fetch--- Cycles: 1161 Stalls:530
Fetch--- Branches: 96 Mispredicts:0


Cycles: 1162  Stalls: 52  Bubbles: 480
Branches: 96  Mispredicts: 0  BTB_Loads: 0

SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}

######################################################################
####
#
StrongArmTest 1.4 ../../object_files/fib-core.tr \
    {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
TRACE FILE IS:../../object_files/fib-core.tr
Fetch--- Cycles: 313 Stalls:93
Fetch--- Branches: 22 Mispredicts:0

Fetch--- Cycles: 314 Stalls:93
Fetch--- Branches: 22 Mispredicts:0

Fetch--- Cycles: 315 Stalls:93
Fetch--- Branches: 22 Mispredicts:0


Cycles: 316  Stalls: 71  Bubbles: 24
Branches: 22  Mispredicts: 0  BTB_Loads: 0

SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}

######################################################################
####
#
StrongArmTest 1.5 ../../object_files/fib_mem.tr \
    {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
TRACE FILE IS:../../object_files/fib_mem.tr
Fetch--- Cycles: 41 Stalls:21
Fetch--- Branches: 1 Mispredicts:0

Fetch--- Cycles: 42 Stalls:21
Fetch--- Branches: 1 Mispredicts:0

Fetch--- Cycles: 43 Stalls:21
Fetch--- Branches: 1 Mispredicts:0


Cycles: 44  Stalls: 2  Bubbles: 21
Branches: 1  Mispredicts: 0  BTB_Loads: 0

SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}

######################################################################
####
#
StrongArmTest 1.6 ../../object_files/fib.tr \
    {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
TRACE FILE IS:../../object_files/fib.tr
Fetch--- Cycles: 1236 Stalls:544
Fetch--- Branches: 105 Mispredicts:0

Fetch--- Cycles: 1237 Stalls:544
Fetch--- Branches: 105 Mispredicts:0

Fetch--- Cycles: 1238 Stalls:544
Fetch--- Branches: 105 Mispredicts:0


Cycles: 1239  Stalls: 48  Bubbles: 498
Branches: 105  Mispredicts: 0  BTB_Loads: 0

SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}

######################################################################
####
#
StrongArmTest 1.7 ../../object_files/mults-core.tr \
    {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
TRACE FILE IS:../../object_files/mults-core.tr
Mult:1
Mult:1
Mult:1
Mult:1
Fetch--- Cycles: 42 Stalls:11
Fetch--- Branches: 0 Mispredicts:0

Mult:1
Fetch--- Cycles: 43 Stalls:11
Fetch--- Branches: 0 Mispredicts:0

Fetch--- Cycles: 45 Stalls:12
Fetch--- Branches: 0 Mispredicts:0


Cycles: 46  Stalls: 7  Bubbles: 7
Branches: 0  Mispredicts: 0  BTB_Loads: 0

SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}

######################################################################
####
#
StrongArmTest 1.8 ../../object_files/test1-core.tr \
    {{Set execution order of processes
Begin setting up connections
- processes to media
- media to media
- processes to statemedia
- media to statemedia
TRACE FILE IS:../../object_files/test1-core.tr
Fetch--- Cycles: 0 Stalls:0
Fetch--- Branches: 0 Mispredicts:0


Cycles: 1  Stalls: 0  Bubbles: 1
Branches: 0  Mispredicts: 0  BTB_Loads: 0

SystemC: simulation stopped by user.



                    ALL RIGHTS RESERVED}}