/*
  Copyright (c) 2003-2004 The Regents of the University of California.
  All rights reserved.

  Permission is hereby granted, without written agreement and without
  license or royalty fees, to use, copy, modify, and distribute this
  software and its documentation for any purpose, provided that the
  above copyright notice and the following two paragraphs appear in all
  copies of this software and that appropriate acknowledgments are made
  to the research of the Metropolis group.

  IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
  ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
  THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
  SUCH DAMAGE.

  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
  PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
  CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
  ENHANCEMENTS, OR MODIFICATIONS.


  METROPOLIS_COPYRIGHT_VERSION_1
  COPYRIGHTENDKEY

*/
#include "GlobalTime.h"
#include "process.h"
#include "programcounter.h"

GlobalTime::GlobalTime(sc_module_name name) : quantity(name)
{
    _gtime = 0;
    _gtimeRequested = 0;
    _stable = false;
}

GlobalTime::GlobalTime(sc_module_name name, DUMMY_CTOR_ARG _arg, int* index) : quantity(name)
{
    _gtime = 0;
    _gtimeRequested = 0;
    _stable = false;
}


GlobalTime::~GlobalTime() {
}

void GlobalTime::request(process *caller, event *e, RequestClass *rc) {
    event *pFound = NULL;

    int i;
    for (i = 0; i < _requests.size() && !pFound; i++) {
        //        request_t *pRequest = _requests[i];
        //        if (pRequest->_e->getProcess() == e->getProcess() &&
        //                *(pRequest->_e->getAction()) == *(e->getAction()))
        //            pFound = pRequest->_e;
        if (_requests[i]->_e->equals(e)) {
            pFound = _requests[i]->_e;
            break;
	}
    } // end for

    if (pFound) {
	if (_requests[i] ->_rc!= NULL) delete _requests[i] ->_rc;
	_requests[i]->_rc = (GlobalTimeRequestClass*) rc;
        //_requests.push_back(new request_t(pFound, (GlobalTimeRequestClass *)rc));
    } else
        _requests.push_back(new request_t(e, (GlobalTimeRequestClass *)rc));
}

void GlobalTime::requestI(process *caller, event *e, GlobalTimeRequestClass *rc) {
    event *pFound = NULL;

    int i;
    for (i = 0; i < _requests.size() && !pFound; i++) {
        //        request_t *pRequest = _requests[i];
        //        if (pRequest->_e->getProcess() == e->getProcess() &&
        //                *(pRequest->_e->getAction()) == *(e->getAction()))
        //            pFound = pRequest->_e;
        if (_requests[i]->_e->equals(e)) {
            pFound = _requests[i]->_e;
            break;
	}
    } // end for

    //convert time interval to absolute time
    ((GlobalTimeRequestClass*)rc)->_gtime += _gtime;

    if (pFound) {
        if (_requests[i] ->_rc!= NULL) delete _requests[i] ->_rc;
	_requests[i]->_rc = (GlobalTimeRequestClass*) rc;
        //_requests.push_back(new request_t(pFound, (GlobalTimeRequestClass *)rc));
    } else
        _requests.push_back(new request_t(e, (GlobalTimeRequestClass *)rc));
}

//void GlobalTime::resolve() {
void GlobalTime::resolve(process *caller) {
    _gtimeRequested = LONG_MAX;
    _stable = true;

    for (int i = 0; i < _requests.size(); i++) {
        request_t *pRequest = _requests[i];
        if (pRequest->_rc->_gtime == NOW)
            _gtimeRequested = _gtime;
        else if (pRequest->_rc->_gtime < _gtimeRequested &&
                pRequest->_rc->_gtime >= _gtime)
            _gtimeRequested = pRequest->_rc->_gtime;
    } // end for

    for (int i = 0; i < _requests.size(); i++) {
        request_t *pRequest = _requests[i];
        if (pRequest->_rc->_gtime != NOW && (pRequest->_rc->_gtime > _gtimeRequested || pRequest->_rc->_gtime < _gtime)) {
            if (pRequest->_e->getProcess()->pc->setMustNotDo(pRequest->_e)) {
                _stable = false;
            } // end if
            //            if (pRequest->_e->getProcess()->pc->SchedState != SchedStateVal::DONTRUN) {
            //                _stable = false;
            //                pRequest->_e->getProcess()->pc->SchedState = SchedStateVal::DONTRUN;
            //            } // end if
        } // end if
    } // end for
}

//bool GlobalTime::stable() {
bool GlobalTime::stable(process *caller) {
    return _stable;
}

//void GlobalTime::postcond() {
void GlobalTime::postcond(process *caller) {
    if (_gtimeRequested == LONG_MAX) return;

    _gtime = _gtimeRequested;

    for (int i = 0; i < _requests.size(); ) {
        request_t *pRequest = _requests[i];
        //        if (pRequest->_e->getProcess()->pc->SchedState == SchedStateVal::RUN || pRequest->_e->getProcess()->pc->SchedState == SchedStateVal::TESTSCHEDULING) {
        if (pRequest->_e->getProcess()->pc->setMustDo(pRequest->_e)) {
            GlobalTimeRequestClass *pRc = new GlobalTimeRequestClass(_gtime);
            pRequest->_e->addQuantityRequestClass((quantity *)this, pRc);

	    if (_requests[i]->_rc!= NULL) delete _requests[i] ->_rc;
            _requests.erase(_requests.begin() + i);
            delete pRequest;
        } else
            i++;
    } // end for
}

//double GlobalTime::A(event *e, int i) {
double GlobalTime::A(process *caller, event *e, int i) {
    GlobalTimeRequestClass *pRc = (GlobalTimeRequestClass *)e->getQuantityRequestClass(this, i);
    if (!pRc) return _gtime;
    return pRc->_gtime;
}
