/*
  Copyright (c) 2003-2004 The Regents of the University of California.
  All rights reserved.

  Permission is hereby granted, without written agreement and without
  license or royalty fees, to use, copy, modify, and distribute this
  software and its documentation for any purpose, provided that the
  above copyright notice and the following two paragraphs appear in all
  copies of this software and that appropriate acknowledgments are made
  to the research of the Metropolis group.

  IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
  ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
  THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
  SUCH DAMAGE.

  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
  PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
  CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
  ENHANCEMENTS, OR MODIFICATIONS.


  METROPOLIS_COPYRIGHT_VERSION_1
  COPYRIGHTENDKEY

*/
/*
 * HeaderCodegen.java
 *
 * Created on May 5, 2003, 2:06 PM
 */

package metropolis.metamodel.backends.systemc;

import metropolis.metamodel.MetaModelStaticSemanticConstants;
import metropolis.metamodel.Modifier;
import metropolis.metamodel.TNLManip;
import metropolis.metamodel.TreeNode;
import metropolis.metamodel.nodetypes.AbsentTreeNode;
import metropolis.metamodel.nodetypes.ArrayTypeNode;
import metropolis.metamodel.nodetypes.BlackboxNode;
import metropolis.metamodel.nodetypes.ClassDeclNode;
import metropolis.metamodel.nodetypes.ConstraintBlockNode;
import metropolis.metamodel.nodetypes.ConstructorDeclNode;
import metropolis.metamodel.nodetypes.InterfaceDeclNode;
import metropolis.metamodel.nodetypes.MediumDeclNode;
import metropolis.metamodel.nodetypes.MethodDeclNode;
import metropolis.metamodel.nodetypes.NetlistDeclNode;
import metropolis.metamodel.nodetypes.ParameterDeclNode;
import metropolis.metamodel.nodetypes.PortDeclNode;
import metropolis.metamodel.nodetypes.ProcessDeclNode;
import metropolis.metamodel.nodetypes.QuantityDeclNode;
import metropolis.metamodel.nodetypes.SMDeclNode;
import metropolis.metamodel.nodetypes.SchedulerDeclNode;
import metropolis.metamodel.nodetypes.TypeNameNode;
import metropolis.metamodel.nodetypes.VarInitDeclNode;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 *
 * @author  Daniele Gasperini
 */
public class HeaderCodegen {

    private SystemCCodegenVisitor _sysC = null;

    private String _filename = null;
    private String _mmmFileName = null;
    private Vector _declaredClasses = new Vector();

    private LinkedList _code = new LinkedList();
    private Vector _forwardDecl = new Vector();
    private Vector _includeDecl = new Vector();
    private Vector _staticDecl = new Vector();      // static variables
    private int _indentLevel = 0;

    private boolean _hasSuperProcess;
    private boolean _hasSCProcess;

    /** Creates a new instance of HeaderCodegen */
    public HeaderCodegen(SystemCCodegenVisitor sysC) {
        _sysC = sysC;
    }

    public boolean hasSuperProcess() { return _hasSuperProcess; }
    public boolean hasSCProcess() { return _hasSCProcess; }

    public void setFilename(String filename) {
        _filename = filename;
    } // end setFilename

    public void setMMMFilename(String filename) {
        _mmmFileName = filename;
    } // end setMMMFilename

    public String getFilename() {
        return _filename;
    } // end getFilename

    public String getMMMFilename() {
        return _mmmFileName;
    } // end getMMMFilename

    public boolean containsClassDeclaration(String name) {
        for (int i = 0; i < _declaredClasses.size(); i++) {
            if (((String)_declaredClasses.get(i)).equals(name)) return true;
        } // end for
        return false;
    } // end containsClassDeclaration

    public LinkedList getCode() {
        return _code;
    } // end getCode

    public String getForwardDeclString() {
        String result = "";
        for (int i = 0; i < _forwardDecl.size(); i++)
            result += "class " + _forwardDecl.get(i) + ";\n";
        return result;
    } // end getForwardDecl

    public Vector getIncludeDecl() {
        return _includeDecl;
    } // end getIncludeDecl

    public void EliminateMetamodelLangFromIncludeDecl() {
        String inc;
        for (int i = 0; i < _includeDecl.size(); i++) {
            inc = (String) _includeDecl.get(i);
            if (inc.indexOf("metro/lib/metamodel/lang")>-1) {
                _includeDecl.remove(i);
                i--;
            }
        }
    } // end EliminateMetamodelLangFromIncludeDecl

    public String getIncludeDeclString() {
        String result = "";
        for (int i = 0; i < _includeDecl.size(); i++)
            result += "#include \"" + _includeDecl.get(i) + "\"\n";

        return result;
    } // end getIncludeDeclString

    public String getStaticDeclString() {
        String result = "";
        for (int i = 0; i < _staticDecl.size(); i++)
            result += _staticDecl.get(i) + ";\n";

        return result;
    } // end getStaticDeclString

    public void replaceIncludeDecl(int index, String classFile) {
        _includeDecl.remove(index);
        if (classFile != null) _includeDecl.add(index, classFile);
    } // end replaceIncludeDecl

    public void compactIncludeDecl() {
        for (int i = 0; i < _includeDecl.size(); i++) {
            String s1 = (String)_includeDecl.get(i);
            for (int j = i + 1; j < _includeDecl.size(); j++) {
                String s2 = (String)_includeDecl.get(j);
                if (s1.equals(s2)) {
                    _includeDecl.remove(j);
                    break;
                } // end if
            } // end for
        } // end for
    } // end replaceIncludeDecl

    public void addDeclaredClass(LinkedList l) {
        addDeclaredClass(SystemCCodegenVisitor._stringListToString(l));
    } // end addForwardDecl

    public void addDeclaredClass(String s) {
        _declaredClasses.add(s);
    } // end addForwardDecl

    public void addForwardDecl(LinkedList l) {
        addForwardDecl(SystemCCodegenVisitor._stringListToString(l));
    } // end addForwardDecl

    public void addForwardDecl(String s) {
        for (int i = 0; i < _includeDecl.size(); i++) {
            if (s.equals(_includeDecl.get(i))) return;
        } // end for

        for (int i = 0; i < _forwardDecl.size(); i++) {
            if (s.equals(_forwardDecl.get(i))) return;
        } // end for
        _forwardDecl.add(s);
    } // end addForwardDecl

    public void addIncludeDecl(LinkedList l) {
        addIncludeDecl(SystemCCodegenVisitor._stringListToString(l));
    } // end addForwardDecl

    public void addIncludeDecl(String s) {
        for (int i = 0; i < _forwardDecl.size(); i++) {
            if (s.equals(_forwardDecl.get(i))) {
                _forwardDecl.remove(i);
                break;
            } // end if
        } // end for

        for (int i = 0; i < _includeDecl.size(); i++) {
            if (s.equals(_includeDecl.get(i))) return;
        } // end for
        _includeDecl.add(s);
    } // end addForwardDecl

    public void visitBlackboxNode(BlackboxNode node, LinkedList args) {
	_code.addLast(node.getCode());
    }

    public void visitPortDeclNode (PortDeclNode node, LinkedList args) {
	indent();
	_code.addLast("sc_port <");
	LinkedList defType = (LinkedList) node.getDefType().accept(_sysC, args);
	LinkedList portName = (LinkedList) node.getName().accept(_sysC, args);
	String portType = SystemCCodegenVisitor._stringListToString(defType).trim();
	String pointer = "";
	int index = portType.indexOf("*");
	if (index > 0) {
            pointer = portType.substring(index);
            portType= portType.substring(0, index);
	} // end if
	int dim = 0;
	for (int i = 0; i < pointer.length(); i++) {
            if (pointer.charAt(i)=='*') dim++;
        } // end for

	_code.addLast(portType);
	_code.addLast(" > ");

	for (int i = 0; i < dim; i++) _code.addLast("*");
	_code.addLast(" ");
	_code.addLast(portName);
	_code.addLast(";\n");

	addForwardDecl(portType);
    } // end visitPortDeclNode

    public void visitProcessDeclNodeBegin (ProcessDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        addForwardDecl(className);
        addDeclaredClass(className);

        args.set(1, className);

	List template = TNLManip.traverseList(_sysC, args, node.getParTypeNames());
	if (!template.isEmpty()) {
	    _code.addLast("\n");
            indent();
    	    _code.addLast("template (class ");
	    _code.addLast(SystemCCodegenVisitor._separateList(template, ", class "));
	    _code.addLast(")");
	} // end if

	_code.addLast("\n");
	indent();
	//retList.addLast(Modifier.toString(node.getModifiers()));
	_code.addLast("class ");
	_code.addLast(className);
	if (node.getSuperClass() != AbsentTreeNode.instance) {
            _code.addLast(" : public ");
            LinkedList superClassName = (LinkedList)node.getSuperClass().accept(_sysC,args);
            _code.addLast(superClassName);
            addIncludeDecl(superClassName);
            _hasSuperProcess = true;
	} else {
            _code.addLast(" : public process ");
            _hasSuperProcess = false;
	} // end if

	_code.addLast(" {\n");
	indent();
	_code.addLast("public:\n");
        // add destructor
        //indent();
        //_code.addLast("~");
        //_code.addLast(className);
        //_code.addLast("() {}\n");

        _hasSCProcess = false;
    } // end visitProcessDeclNode

    public void visitProcessDeclNodeEnd (ProcessDeclNode node, LinkedList args) {
	indent();
	_code.addLast("\n};\n");
    } // end visitProcessDeclNodeEnd

    public void visitMediumDeclNodeBegin (MediumDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        LinkedList superClassName = null;
        addForwardDecl(className);
        addDeclaredClass(className);

        args.set(1, className);

	List template = TNLManip.traverseList(_sysC, args, node.getParTypeNames());
	if (!template.isEmpty()) {
            _code.addLast("\n");
            indent();
    	    _code.addLast("template (class ");
	    _code.addLast(SystemCCodegenVisitor._separateList(template, ", class "));
	    _code.addLast(")");
	} // end if

	_code.addLast("\n");
	indent();
	_code.addLast("class ");
	_code.addLast(className);
	if (node.getSuperClass()!=AbsentTreeNode.instance) {
            _code.addLast(" : public ");
            superClassName = (LinkedList)node.getSuperClass().accept(_sysC, args);
            _code.addLast(superClassName);
            addIncludeDecl(superClassName);
	} else {
	    _code.addLast(" : public medium");
	}

	List retValue = TNLManip.traverseList(_sysC, args, node.getInterfaces());
	if (!retValue.isEmpty()) {
            _code.addLast(", public ");

            Iterator stringListItr = retValue.iterator();

            while (stringListItr.hasNext()) {
                LinkedList intfcClassName = (LinkedList)stringListItr.next();
                _code.addLast(intfcClassName);
                addIncludeDecl(intfcClassName);
                if (stringListItr.hasNext()) _code.addLast(", public ");
            } // end while
	} // end if

	_code.addLast("{\n");
        indent();
	_code.addLast("public:\n");

        // add destructor
        //indent();
        //_code.addLast("~");
        //_code.addLast(className);
        //_code.addLast("() {}\n");
    } // end visitMediumDeclNodeBegin

    public void visitMediumDeclNodeEnd (MediumDeclNode node, LinkedList args) {
	indent();
	_code.addLast("\n};\n");
    } // end visitMediumDeclNodeEnd

    public void visitSchedulerDeclNodeBegin (SchedulerDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        addForwardDecl(className);
        addDeclaredClass(className);

        args.set(1, className);

	List template = TNLManip.traverseList(_sysC, args, node.getParTypeNames());
	if (!template.isEmpty()) {
	    _code.addLast("\n");
            indent();
    	    _code.addLast("template (class ");
	    _code.addLast(SystemCCodegenVisitor._separateList(template, ", class "));
	    _code.addLast(")");
	} // end if

	_code.addLast("\n");
	indent();
	_code.addLast("class ");
	_code.addLast(className);
	if (node.getSuperClass() != AbsentTreeNode.instance) {
            _code.addLast(" : public ");
            LinkedList superClassName = (LinkedList)node.getSuperClass().accept(_sysC, args);
            _code.addLast(superClassName);
            addIncludeDecl(superClassName);
	} else {
	    _code.addLast(" : public scheduler");
	} // end if

	_code.addLast(" {\n");
	indent();
	_code.addLast("public:\n");
        // add destructor
        //indent();
        //_code.addLast("~");
        //_code.addLast(className);
        //_code.addLast("() {}\n");
    } // end visitSchedulerDeclNodeBegin

    public void visitSchedulerDeclNodeEnd (SchedulerDeclNode node, LinkedList args) {
	indent();
	_code.addLast("\n};\n");
    } // end visitSchedulerDeclNodeEnd

    public void visitSMDeclNodeBegin (SMDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        addForwardDecl(className);
        addDeclaredClass(className);

        args.set(1, className);

	List template = TNLManip.traverseList(_sysC, args, node.getParTypeNames());
	if (!template.isEmpty()) {
            _code.addLast("\n");
            indent();
    	    _code.addLast("template (class ");
	    _code.addLast(SystemCCodegenVisitor._separateList(template, ", class"));
	    _code.addLast(")");
	} // end if

	_code.addLast("\n");
        indent();
        _code.addLast("class ");
        _code.addLast(className);

        if (node.getSuperClass() != AbsentTreeNode.instance) {
            _code.addLast(" : public ");
            LinkedList superClassName = (LinkedList)node.getSuperClass().accept(_sysC, args);
            _code.addLast(superClassName);
            addIncludeDecl(superClassName);
        } else {
            _code.addLast(" : public statemedium ");
	} // end if

        List retValue = TNLManip.traverseList(_sysC, args, node.getInterfaces());
        if (!retValue.isEmpty()) {
            _code.addLast(", public ");

            Iterator stringListItr = retValue.iterator();

            while (stringListItr.hasNext()) {
                LinkedList superClassName = (LinkedList)stringListItr.next();
                _code.addLast(superClassName);
                addIncludeDecl(superClassName);
                if (stringListItr.hasNext()) _code.addLast(", public ");
            } // end while
        } // end if

        _code.addLast(" {\n");
        indent();
        _code.addLast("public:\n");
        // add destructor
        //indent();
        //_code.addLast("~");
        //_code.addLast(className);
        //_code.addLast("() {}\n");
    } // end visitSMDeclNodeBegin

    public void visitSMDeclNodeEnd (SMDeclNode node, LinkedList args) {
	indent();
	_code.addLast("\n};\n");
    } // end visitSMDeclNodeEnd

    public void visitNetlistDeclNodeBegin (NetlistDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        addForwardDecl(className);
        addDeclaredClass(className);

        args.set(1, className);

        //Fix me: refer to ProcessDeclNode to make this part work completely
	List template = TNLManip.traverseList(_sysC, args, node.getParTypeNames());
	if (!template.isEmpty()) {
            _code.addLast("\n");
            indent();
    	    _code.addLast("template (class ");
	    _code.addLast(SystemCCodegenVisitor._separateList(template, ", class"));
	    _code.addLast(")");
	} // end if

	_code.addLast("\n");
        indent();
        _code.addLast("class ");
        _code.addLast(className);

        if (node.getSuperClass() != AbsentTreeNode.instance) {
            _code.addLast(" : public ");
            LinkedList superClassName = (LinkedList)node.getSuperClass().accept(_sysC, args);
            _code.addLast(superClassName);
            addIncludeDecl(superClassName);
        } else {
            _code.addLast(" : public netlist");
        } // end if

        _code.addLast(" {\n");
        indent();
        _code.addLast("public:\n");
        // add destructor
        //indent();
        //_code.addLast("~");
        //_code.addLast(className);
        //_code.addLast("() {};\n");
    } // end visitNetlistDeclNodeBegin

    public void visitNetlistDeclNodeEnd (NetlistDeclNode node, LinkedList args) {
        //force one constructor with args (sc_module_name)
        indent();
	_code.addLast(node.getName().accept(_sysC, args));
	_code.addLast("(sc_module_name _name) : ");
	if (node.getSuperClass() instanceof AbsentTreeNode)
            _code.addLast("netlist(_name) {};\n");
	else {
            _code.addLast(node.getSuperClass().accept(_sysC, args));
            _code.addLast("(_name) {};\n");
        } // end if

        indent();
	_code.addLast("};\n");
    } // end visitNetlistDeclNodeEnd

    public void visitClassDeclNodeBegin (ClassDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        addForwardDecl(className);
        addDeclaredClass(className);

        args.set(1, className);

	// visit type parameters
	List template = TNLManip.traverseList(_sysC, args, node.getParTypeNames());
	if (!template.isEmpty()) {
	    _code.addLast("\n");
            indent();
            _code.addLast("template (class ");
	    _code.addLast(SystemCCodegenVisitor._separateList(template, ", class"));
	    _code.addLast(")");
	} // end if

	_code.addLast("\n");
        indent();
	_code.addLast("class ");
	_code.addLast(className);

	_code.addLast(" ");

	TreeNode superClass = node.getSuperClass();
	_code.addLast(" : public ");
	if (superClass != AbsentTreeNode.instance) {
            LinkedList superClassName = (LinkedList)node.getSuperClass().accept(_sysC, args);
	    _code.addLast(superClassName);
            addIncludeDecl(superClassName);
        } else
	    _code.addLast("Object");

        _code.addLast(" ");

	// visit interfaces
	List retValue = TNLManip.traverseList(_sysC, args, node.getInterfaces());
	if (!retValue.isEmpty()) {
            if (superClass == AbsentTreeNode.instance)
                _code.addLast(" : public ");
            else
		_code.addLast(", public ");

            Iterator stringListItr = retValue.iterator();

            while (stringListItr.hasNext()) {
                LinkedList superClassName = (LinkedList)stringListItr.next();
                _code.addLast(superClassName);
                addIncludeDecl(superClassName);
                if (stringListItr.hasNext()) _code.addLast(", public ");
            } // end while
	} // end if

	_code.addLast("{\n");
        indent();
	_code.addLast("public:\n");
        // add destructor
        //indent();
        //_code.addLast("~");
        //_code.addLast(className);
        //_code.addLast("() {}\n");
    } // end visitClassDeclNodeBegin

    public void visitClassDeclNodeEnd (ClassDeclNode node, LinkedList args) {
	indent();
	_code.addLast("\n};\n");
    } // end visitClassDeclNodeEnd

    public void visitInterfaceDeclNodeBegin (InterfaceDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        addForwardDecl(className);
        addDeclaredClass(className);

        args.set(1, className);

        List template = TNLManip.traverseList(_sysC, args, node.getParTypeNames());
	if (!template.isEmpty()) {
            _code.addLast("\n");
            indent();
            _code.addLast("template (class ");
            _code.addLast(SystemCCodegenVisitor._separateList(template, ", class"));
	    _code.addLast(")");
	} // end if

	_code.addLast("\n");
	indent();
	_code.addLast("class ");
	_code.addLast(className);
    	_code.addLast(" : ");

        // visit interfaces
        List retValue = TNLManip.traverseList(_sysC, args, node.getInterfaces());
        if (!retValue.isEmpty()) {
            Iterator ListItr = retValue.iterator();
            boolean hasParent = false;

            while (ListItr.hasNext()) {
                String str = (String) ((LinkedList) (ListItr.next())).getFirst();
                if (!str.equals("Port")) {
                    if (hasParent) _code.addLast(", ");
                    if (str.equals("StateMediumSched") || str.equals("StateMediumProc") || str.equals("QuantityManager"))
                        _code.addLast("virtual ");
                    _code.addLast("public "+str);
                    hasParent = true;
                } // end if
            } // end while
            if (!hasParent) _code.addLast("virtual public sc_interface");
        } else {
            _code.addLast("virtual public sc_interface");
        } // end if

        _code.addLast("{\n");
        _code.addLast("public:\n");
        // add destructor
        //indent();
        //_code.addLast("virtual ~");
        //_code.addLast(className);
        //_code.addLast("() {}\n");
    } // end visitInterfaceDeclNodeBegin

    public void visitInterfaceDeclNodeEnd (InterfaceDeclNode node, LinkedList args) {
	indent();
	_code.addLast("\n};\n");
    } // end visitInterfaceDeclNodeEnd

    public void visitQuantityDeclNodeBegin(QuantityDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        addForwardDecl(className);
        addDeclaredClass(className);

        args.set(1, className);

	List template = TNLManip.traverseList(_sysC, args, node.getParTypeNames());
	if (!template.isEmpty()) {
	    _code.addLast("\n");
            indent();
    	    _code.addLast("template (class ");
	    _code.addLast(SystemCCodegenVisitor._separateList(template, ", class "));
	    _code.addLast(")");
	} // end if

	_code.addLast("\n");
	indent();
	//retList.addLast(Modifier.toString(node.getModifiers()));
	_code.addLast("class ");
	_code.addLast(className);
	if (node.getSuperClass() != AbsentTreeNode.instance) {
            _code.addLast(" : public ");
            LinkedList superClassName = (LinkedList)node.getSuperClass().accept(_sysC,args);
            _code.addLast(superClassName);
            addIncludeDecl(superClassName);
	} else {
            _code.addLast(" : public quantity ");
	} // end if

	List retValue = TNLManip.traverseList(_sysC, args, node.getInterfaces());
	if (!retValue.isEmpty()) {
            Iterator stringListItr = retValue.iterator();

            while (stringListItr.hasNext()) {
                String intfcClassName = SystemCCodegenVisitor._stringListToString((LinkedList)stringListItr.next());
                //LinkedList intfcClassName = (LinkedList)stringListItr.next();
                if (!intfcClassName.equals("QuantityManager")) {
                    _code.addLast(", public ");
                    _code.addLast(intfcClassName);
                    addIncludeDecl(intfcClassName);
                }
            } // end while
	} // end if


	_code.addLast(" {\n");
	indent();
	_code.addLast("public:\n");
        // add destructor
        //indent();
        //_code.addLast("~");
        //_code.addLast(className);
        //_code.addLast("() {}\n");

    } // end visitQuantityDeclNode

    public void visitQuantityDeclNodeEnd (QuantityDeclNode node, LinkedList args) {
	indent();
	_code.addLast("\n};\n");
    } // end visitQuantityDeclNodeEnd

    public void visitMethodDeclNode(MethodDeclNode node, LinkedList args) {
	String type = args.get(0).toString();
        boolean parent_was_object = wasParentObject(node.getParent().classID());

	_code.addLast("\n");

	boolean isStatic = false;
	String mod = Modifier.toString(node.getModifiers());
	StringTokenizer st = new StringTokenizer(mod);
	while (st.hasMoreTokens()) {
	    String modifier = st.nextToken();
	    if (modifier.equals("final")) {
	        if (parent_was_object) _code.addLast("const ");
	    } else if (modifier.equals("static")) {
	        isStatic = true;
	        if (parent_was_object) _code.addLast(modifier + " ");
	    }
	    // public, protected, private, abstract ignored
	}

	if (!isStatic) {
	    if (parent_was_object) _code.addLast("virtual ");
	} // end if

	_code.addLast(node.getReturnType().accept(_sysC,args));

	TreeNode baseType = node.getReturnType();
	while (baseType instanceof ArrayTypeNode)
	    baseType = ((ArrayTypeNode)baseType).getBaseType();

	if (baseType instanceof TypeNameNode) {
	    if (parent_was_object) _code.addLast("*");
	} // end if

	_code.addLast(" ");

	if (parent_was_object) {
	    _code.addLast(node.getName().accept(_sysC, args));
	    _code.addLast("(");
	} // end if

	LinkedList par = SystemCCodegenVisitor._commaList(TNLManip.traverseList(_sysC, args, node.getParams()));
	if (!(type.equals("Process") && node.getName().getIdent().toString().equals("thread"))) {
	    _code.addLast("process * caller");
	    if (SystemCCodegenVisitor._stringListToString(par).trim().length() > 0) _code.addLast(", ");
	} // end if

	if (parent_was_object) {
	    _code.addLast(par);
	    _code.addLast(")");
	    if (node.getBody() instanceof AbsentTreeNode) {
	        if (type.equals("Interface")) _code.addLast("= 0");
	    } // end if

	    _code.addLast(";\n");

	} // end if

    } // end visitMethodDeclNode

    public void visitConstraintBlockNode(ConstraintBlockNode node, LinkedList args) {
        TreeNode parent = node.getParent();
        while (!(parent instanceof ConstructorDeclNode || parent instanceof MethodDeclNode))
            parent = parent.getParent();
        if (parent instanceof ConstructorDeclNode) {
            parent = ((ConstructorDeclNode)parent).getParent();
        } else if (parent instanceof MethodDeclNode) {
            parent = ((MethodDeclNode)parent).getParent();
        }

        boolean parent_was_object = wasParentObject(parent.classID());

        if (parent_was_object) {
            _code.add("\n");
            indent();
            _code.add("virtual void synch(int);\n\n");
            indent();
            _code.add("virtual void ltl();\n");
        } // end if
    } // end visitConstraintBlockNode

    public void visitConstructorDeclNode(ConstructorDeclNode node, LinkedList args) {
        boolean parent_was_object = wasParentObject(node.getParent().classID());
        LinkedList constructorName = (LinkedList)node.getName().accept(_sysC, args);

	if (!_hasSCProcess && args.get(0).equals("Process")) {
            indent();
            _code.addLast("SC_HAS_PROCESS(");
            _code.addLast(constructorName);
            _code.addLast(");\n");
            _hasSCProcess = true;
	} // end if

	if (parent_was_object && !_hasSCProcess) _code.addLast("\n");

	//constructors for classes
	if (parent_was_object) _code.addLast(constructorName);

        LinkedList par = SystemCCodegenVisitor._commaList(TNLManip.traverseList(_sysC, args, node.getParams()));
	if (args.get(0).equals("Class")) {
            if (parent_was_object) _code.addLast("(");
	} else {
            if (parent_was_object) _code.addLast("(sc_module_name _name");
            if (SystemCCodegenVisitor._stringListToString(par).trim().length() > 0) {
                if (parent_was_object) _code.addLast(", ");
            } // end if
	} // end if

	if (parent_was_object) {
            _code.addLast(par);
            _code.addLast(");\n");
	} // end if
    } // end visitConstructorDeclNode

    public void visitVarInitDeclNode(VarInitDeclNode node, LinkedList args) {
        boolean parent_was_object = wasParentObject(node.getParent().classID());

	TreeNode baseType = node.getDefType();

	if (parent_was_object) {
            boolean bStatic = false;
            boolean bConst = false;

            indent();

            LinkedList staticDecl = new LinkedList();

            String mod = Modifier.toString(node.getModifiers());
            StringTokenizer st = new StringTokenizer(mod);
            while (st.hasMoreTokens()) {
                String modifier = st.nextToken();
                if (modifier.equals("final")) {
                    _code.addLast("const ");
                    staticDecl.addLast("const ");
                    bConst = true;
                } else if (modifier.equals("static") || modifier.equals("abstract")) {
                    _code.addLast(modifier + " ");
                    bStatic = true;
                } // end if
                //public, protected, private ignored
            } // end while


            LinkedList typeName = (LinkedList)node.getDefType().accept(_sysC, args);
            _code.addLast(typeName);

            staticDecl.addLast(typeName);

            //if it is user defined type then it has to be a pointer
            while (baseType instanceof ArrayTypeNode)
                baseType = ((ArrayTypeNode)baseType).getBaseType();
            if (baseType instanceof TypeNameNode) {
                _code.addLast("*");
                staticDecl.addLast("*");
            } // end if

            _code.addLast(" " + node.getName().getIdent());
            if (bConst) {
                if (node.getInitExpr() != AbsentTreeNode.instance) {
                    _code.addLast(" = ");
                    _code.addLast(node.getInitExpr().accept(_sysC, args));
                } // end if
            } else if (bStatic) {
                staticDecl.addLast(" ");
                staticDecl.addLast(args.get(1));
                staticDecl.addLast("::" + node.getName().getIdent() + " = ");
                if (node.getInitExpr() != AbsentTreeNode.instance)
                    staticDecl.addLast(node.getInitExpr().accept(_sysC, args));
                else
                    staticDecl.addLast("0");
                staticDecl.addLast(";\n");

                _staticDecl.add(SystemCCodegenVisitor._stringListToString(staticDecl));
            } // end if
            _code.addLast(";\n");
	} // end if
    } // end _visitVarInitDeclNode

    public void visitParameterDeclNode(ParameterDeclNode node, LinkedList args) {
        boolean parent_was_object = wasParentObject(node.getParent().classID());

	if (parent_was_object) {
            indent();
            _code.addLast(Modifier.toString(node.getModifiers()));
            _code.addLast(node.getDefType().accept(_sysC, args));
            TreeNode baseType = node.getDefType();
            while (baseType instanceof ArrayTypeNode)
                baseType = ((ArrayTypeNode)baseType).getBaseType();
            if (baseType instanceof TypeNameNode) {
                _code.addLast("*");
            } // end if
            _code.addLast(" ");
            _code.addLast(node.getName().accept(_sysC, args));
            _code.addLast(";\n");
	} // end if
    } // end visitParameterDeclNode

    public void dummy_ctor_gen (ClassDeclNode node, LinkedList args) {
        LinkedList className = (LinkedList)node.getName().accept(_sysC, args);
        if (args.get(0).equals("Class")) {
            _code.addLast("\n");
            _code.addLast(className);
            _code.addLast("(DUMMY_CTOR_ARG _arg, int *index);");

            // provide default constructor
            List mem = node.getMembers();
            Iterator memIter = mem.iterator();
            boolean flag = false;
            while (memIter.hasNext()) {
                TreeNode tnode = (TreeNode) memIter.next();
                if (tnode instanceof ConstructorDeclNode) {
                    List par = ((ConstructorDeclNode) tnode).getParams();
                    if (par == null) {
                        flag = true;
                        break;
                    } else if (par.isEmpty()) {
                        flag = true;
                        break;
                    } // end if
                } // end if
            } // end while
            if (!flag) {
                _code.addLast("\n");
                indent();
                _code.addLast(className);
                _code.addLast("() {}\n");
            } // end if
	} else {
            _code.addLast("\n");
            _code.addLast(className);
            _code.addLast("(sc_module_name _name, DUMMY_CTOR_ARG _arg, int *index); ");
        } // end if
    } // end dummy_ctor_gen

    public boolean wasParentObject(int classID) {
	switch (classID) {
        case MetaModelStaticSemanticConstants.QUANTITYDECLNODE_ID:
            return true;
        case MetaModelStaticSemanticConstants.CLASSDECLNODE_ID:
            return true;
        case MetaModelStaticSemanticConstants.PROCESSDECLNODE_ID:
            return true;
        case MetaModelStaticSemanticConstants.MEDIUMDECLNODE_ID:
            return true;
        case MetaModelStaticSemanticConstants.SCHEDULERDECLNODE_ID:
            return true;
        case MetaModelStaticSemanticConstants.SMDECLNODE_ID:
            return true;
        case MetaModelStaticSemanticConstants.INTERFACEDECLNODE_ID:
            return true;
        case MetaModelStaticSemanticConstants.NETLISTDECLNODE_ID:
            return true;
        default:
        }

        return false;
    } // end wasParentObject

    private void indent() {
        String blk = "";
        for (int i = 0; i < _indentLevel; i++)
            blk += " ";
        _code.addLast(blk);
    } // end indent

    //    private void indentInc() {
    //        _indentLevel += 2;
    //    } // end indentInc
    //
    //    private void indentDec() {
    //        _indentLevel -= 2;
    //    } // end indentDec
}
