/*
  Metropolis: Design Environment for Heterogeneus Systems.

  Copyright (c) 1998-2004 The Regents of the University of California.
  All rights reserved.

  Permission is hereby granted, without written agreement and without
  license or royalty fees, to use, copy, modify, and distribute this
  software and its documentation for any purpose, provided that the
  above copyright notice and the following two paragraphs appear in all
  copies of this software and that appropriate acknowledgments are made
  to the research of the Metropolis group.

  IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
  ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
  THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
  SUCH DAMAGE.

  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
  PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
  CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
  ENHANCEMENTS, OR MODIFICATIONS.


  METROPOLIS_COPYRIGHT_VERSION_1
  COPYRIGHTENDKEY
*/

package metropolis.metamodel.backends.systemc;

import metropolis.metamodel.runtime.Event;

/**
 * A class used to represent one variable specified by synch constraints
 * @author   Guang Yang
 * @version  $Id: VarAttribute.java,v 1.9 2004/05/20 04:12:46 cxh Exp $
 */

public class VarAttribute {
    /**
     * The event in whose scope the variable exists
     */
    protected Event _event;

    /**
     * The variable index in the same synch event group
     */
    protected int _indexInSynchGroup;

    /**
     * Indicate whether or not this variable is processed
     * in code generation for comparing equal variables
     */
    protected boolean _processed;

    /**
     * To which VarAttribute this instance is equal to
     */
    protected VarAttribute _equalVar;

    /**
     * Variable name
     */
    protected String _varName;

    /**
     * Variable type
     */
    protected String _type;

    /**
     * Whether or not this variable is nondeterministic
     */
    protected boolean _nondet;

    public VarAttribute(Event event, String varName, String type, boolean nondet) {
        _varName = varName;
        _type = type;
        _nondet = nondet;
        _event = event;
        _equalVar = this;
        _indexInSynchGroup = -1;
        _processed = false;
    }

    /**
     * Get the name of the variable
     * @return Variable name
     */
    public String getVarName() {
        return _varName;
    }

    /**
     * Get the type of the variable
     * @return Variable type
     */
    public String getType() {
        return _type;
    }

    /**
     * Check whether or not a variable is nondeterministic
     * @return whether or not a variable is nondeterministic
     */
    public boolean isNondetVar() {
        return _nondet;
    }
    /**
     * @return Returns the _equalVar.
     */
    public VarAttribute getEqualVar() {
        return _equalVar;
    }

    /**
     * @param var The _equalVar to set.
     */
    public void setEqualVar(VarAttribute var) {
        _equalVar = var;
    }

    /**
     * @return Returns the _event.
     */
    public Event getEvent() {
        return _event;
    }

    /**
     * @param event The _event to set.
     */
    public void setEvent(Event event) {
        _event = event;
    }

    /**
     * @return Returns the _indexInSynchGroup.
     */
    public int getIndexInSynchGroup() {
        return _indexInSynchGroup;
    }

    /**
     * @param inSynchGroup The _indexInSynchGroup to set.
     */
    public void setIndexInSynchGroup(int inSynchGroup) {
        _indexInSynchGroup = inSynchGroup;
    }

    /** Return true if equal.
     */
    public boolean equals(Event event, String varName, String type,
            boolean nondet) {
        return ((event==_event) && (varName.equals(_varName) && (type.equals(_type)) && (nondet==_nondet)));
    }

    /**
     * @return true if this variable is processed in code generation
     * for comparing equal variables.
     */
    public boolean isProcessed() {
        return _processed;
    }

    /**
     * @param processed The _processed to set.
     */
    public void setProcessed(boolean processed) {
        _processed = processed;
    }

}
