/* A utility class for getting information about TreeNodes for debugging.

Metropolis: Design Environment for Heterogeneus Systems.

Copyright (c) 1998-2004 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software and that appropriate acknowledgments are made
to the research of the Metropolis group.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.


METROPOLIS_COPYRIGHT_VERSION_1
COPYRIGHTENDKEY
*/

//////////////////////////////////////////////////////////////////////////
//// DebugUtil
/**
 * A utility class for getting information about <code>TreeNode</code>s
 * for debugging.
 * <p>
 * @author   Allen Hopkins
 * @version  $Id: DebugUtil.java,v 1.6 2004/05/25 20:31:01 allenh Exp $
 */

package metropolis.metamodel.backends.systemc.mmdebug;

import metropolis.metamodel.MetaModelStaticSemanticConstants;
import metropolis.metamodel.TreeNode;
import metropolis.metamodel.nodetypes.NameNode;

import java.io.File;

public class DebugUtil implements MetaModelStaticSemanticConstants {

    ////////////////////////////////////////////////////////////////////////
    ////                public methods

    public static void familyTree(TreeNode node) {
        System.err.println("I am a " + getNodeKind(node));
        System.err.print("...at " + getNodeFileName(node));
        System.err.println(":" + getNodeLineNumber(node));
        System.err.println("My parent is a "
                + getNodeKind(node.getParent()));
        for (TreeNode p = node.getParent().getParent();
             p != null;
             p = p.getParent()) {
            System.err.println(".. whose parent is a "
                    + getNodeKind(p));
            System.err.println("   with next sibling of "
                    + getNodeNextSibling(p));
        }
    }

    /**
     * Return just the basename of the name of the .mmm file this node
     * comes from.
     */
    public static String getNodeFileName(TreeNode node) {
        String fullName = (String)node.getProperty(IDENT_KEY);
        if (fullName == null) {
            return "null";
        } else {
            return fullName.substring(
                    fullName.lastIndexOf(File.separator) + 1);
        }
    }

    /**
     * Return the simple, un-qualified class name of this node.
     */
    public static String getNodeKind(TreeNode node) {
        String fullName = node.getClass().getName();
        return fullName.substring(fullName.lastIndexOf(".") + 1);
    }

    /**
     * Return the name of this node, if it has one.
     */
    public static String getNodeName(TreeNode node) {
        Class nodeClass = node.getClass();
        try {
            java.lang.reflect.Method nameMethod =
                nodeClass.getMethod("getName", null);
            NameNode name = (NameNode)nameMethod.invoke(node, null);
            return name.getIdent();
        } catch (Exception ex) {
            return "";
        }
    }

    /**
     * Return the LINENUMBER_KEY property of this node.
     */
    public static String getNodeLineNumber(TreeNode node) {
        StringBuffer buf = new StringBuffer();
        buf.append(node.getProperty(LINENUMBER_KEY));
        return buf.toString();
    }

    /**
     * Return the NEXT_SIBLING_LINENUMBER_KEY property of this node.
     */
    public static String getNodeNextSibling(TreeNode node) {
        StringBuffer buf = new StringBuffer();
        buf.append(node.getProperty(NEXT_SIBLING_LINENUMBER_KEY));
        return buf.toString();
    }

    /**
     * Return the given node's current line number, and simple source-code
     * file name, in the form "blah.mmm:xx".
     *
     * @param node The <code>TreeNode</code> to get the filename and
     *  line number property for.
     * @return the <code>String</code>, in the form "blah.mmm:xx".
     */
    public static String getCurrentLineNumberString(TreeNode node) {
        return getCurrentLineNumberString(
                node, (Integer)node.getProperty(LINENUMBER_KEY));
    }

    /**
     * Return the given node's current line number, and simple source-code
     * file name, in the form "blah.mmm:xx", where the line number is
     * explicitly supplied.
     *
     * @param node The <code>TreeNode</code> to get the filename for.
     * @param lineNumber The explicit <code>Integer</code> line number.
     * @return the <code>String</code>, in the form "blah.mmm:xx".
     */
    public static String getCurrentLineNumberString(TreeNode node,
            Integer lineNumber) {
        StringBuffer buf = new StringBuffer();
        buf.append(getNodeFileName(node));
        buf.append(":");
        buf.append(lineNumber);
        return buf.toString();
    }
}
