/* A ltl synch constraint in the elaborated network.

Metropolis: Design Environment for Heterogeneus Systems.

Copyright (c) 1998-2004 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software and that appropriate acknowledgments are made
to the research of the Metropolis group.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/


package metropolis.metamodel.runtime;

import java.io.Serializable;
import java.util.Iterator;

//////////////////////////////////////////////////////////////////////////
//// LTLSynch
/**
   An ltl synch constraint in the elaborated network.

   @author   Xi Chen
   @version  $Id: LTLSynch.java,v 1.11 2004/05/20 04:06:50 cxh Exp $
*/
public class LTLSynch extends Constraint implements Serializable {

    ///////////////////////////////////////////////////////////////////
    ////                           constructors                    ////

    /** Build a new ltl synch constraint in the network.
     *  @param kind  the constraint type, LTL, LOC or ELOC.
     */
    public LTLSynch(int kind) {
        this(null, kind, -1);
    }


    /** Build a new ltl synch constraint in the network.
     *  @param container  the node which contains this constraint.
     *  @param kind  the constraint type, LTL, LOC or ELOC.
     *  @param index the sequence number of the constraint.
     */
    public LTLSynch(Object container, int kind, int index) {
        super(container, kind, index);
        _var1 = null;
        _var2 = null;
        _event1 = null;
        _event2 = null;
    }

    ///////////////////////////////////////////////////////////////////
    ////                          public methods                   ////

    /** Set var1.
     *  @param  var The variable need to be set.
     *  @see #getVar1()
     */
    public void setVar1(String var) {
        _var1 = var;
    }

    /** Set var2.
     *  @param  var  the variable need to be set.
     *  @see #getVar2()
     */
    public void setVar2(String var) {
        _var2 = var;
    }

    /** Set event1.
     *  @param event The event reference need to be set.
     *  @see #getEvent1()
     */
    public void setEvent1(Event event) {
        _event1 = event;
    }

    /** Set event2.
     *  @param event the event reference need to be set.
     *  @see #getEvent2()
     */
    public void setEvent2(Event event) {
        _event2 = event;
    }

    /** Get var1.
     *  @return var1.
     *  @see #setVar1(String)
     */
    public String getVar1() {
        return _var1;
    }

    /** Get var2.
     *  @return var2.
     *  @see #setVar2(String)
     */
    public String getVar2() {
        return _var2;
    }

    /** Get event1.
     *  @return event1.
     *  @see #setEvent1(Event)
     */
    public Event getEvent1() {
        return _event1;
    }

    /** Get event2.
     *  @return event2.
     *  @see #setEvent2(Event)
     */
    public Event getEvent2() {
        return _event2;
    }

    /** Get a String providing information about this constraint.
     *  @return A String providing information about this constraint.
     */
    public String show() {
        StringBuffer results = new StringBuffer(super.show());
        String indent = "          ";

        if (_var1 != null && _var2 != null
                && _event1 != null && _event2 != null) {
            results.append(indent + "o Equality\n"
                    + indent + "  " + _var1 + "@(" + _event1.show()
                    + ", i) == " + _var2 + "@(" + _event2.show() + ", i)\n");
        }
        return results.toString();
    }

    /** Check if both event1 and event2 are in the event list.
     *  Do nothing if both event1 and event2 are in the event list,
     *  otherwise throw an exception.
     */
    public void checkEvents() {
        boolean flag1 = false, flag2 = false;

        Iterator events = _events.iterator();
        while (events.hasNext()) {
            Event event = (Event)events.next();
            if (event.equals(_event1)) {
                flag1 = true;
            }
            if (event.equals(_event2)) {
                flag2 = true;
            }
        }

        if (!(flag1 && flag2)) {
            throw new RuntimeException("Error on ltl synch constraint: "
                    + "event references in equality part must appear in "
                    + "the event list.  Either event1: '" + _event1
                    + "' or event2 '" + _event2
                    + "' were not in the event list");
        }
    }

    ///////////////////////////////////////////////////////////////////
    ////              protected variables                ////

    /** Variable 1. */
    protected String _var1;

    /** Variable 2. */
    protected String _var2;

    /** Event 1. */
    protected Event _event1;

    /** Event 2. */
    protected Event _event2;

}
