# Tests for the Compiler class
#
# @Author: Christopher Hylands Brooks
#
# @Version: $Id: Compiler.tcl,v 1.7 2004/06/04 14:57:43 cxh Exp $
#
# @Copyright (c) 2004 The Regents of the University of California.
# All rights reserved.
#
# Permission is hereby granted, without written agreement and without
# license or royalty fees, to use, copy, modify, and distribute this
# software and its documentation for any purpose, provided that the
# above copyright notice and the following two paragraphs appear in all
# copies of this software and that appropriate acknowledgments are made
# to the research of the Metropolis group.
# 
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
# THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
# PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
# CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.
#
# 						PT_COPYRIGHT_VERSION_2
# 						COPYRIGHTENDKEY
#######################################################################

# Tycho test bed, see $TYCHO/doc/coding/testing.html for more information.

# Load up the test definitions.
if {[string compare test [info procs test]] == 1} then {
    source testDefs.tcl
} {}

if {[info procs iterToObjects] == "" } then {
    source [ file join $METRO util testsuite enums.tcl]
}

if {[info procs jdkCapture] == "" } then {
    source [ file join $METRO util testsuite jdktools.tcl]
}

# Uncomment this to get a full report, or set in your Tcl shell window.
# set VERBOSE 1


######################################################################
####
#
test Compiler-1.1 {_printHelp} {
    jdkCaptureErr {
	java::call metropolis.metamodel.test.TestCompiler printHelp
    } output
    list $output
} {{
class metropolis.metamodel.Compiler:
       A compiler for the meta-model language, the internal representation
       used in the Metropolis design framework

Usage: java metropolis.metamodel.Compiler [options] source_files
         
where options include:
   -? -help -h                Display this help message
   -classpath <classpath>     Specify where to find user class files
   -metamodel [-dumpast]      Use the metamodel back-end (default)
                              -dumpast prints a dump of the AST to STDOUT
   -javasim [-top <netinit>] [-java <path>/java] [-javac <path>/javac]
                              Generate java simulation code (semantics 3.0)
                              [javasim not shipped in Metropolis 1.0]
   -cpp                       Generate C++ simulation code
                              [cpp not shipped in Metropolis 1.0]
   -simulator                 Generate java simulation code
                              [simulator not shipped in Metropolis 1.0]
   -systemc [-top <netinit>] [-java <path>/java] [-javac <path>/javac] [-w] [-mmdebug]
                              Generate SystemC simulation code
   -elaborator <netinit>      Compute the structure of the network
   -promela                   Generate Promela/Spin code
   -cfa                       Generate a CFA; Visual, KISS,
                              and RML representations
                              [cfa not shipped in Metropolis 1.0]

}}

######################################################################
####
#
test Compiler-2.1 {_printHelp} {
    jdkCaptureErr {
	java::call metropolis.metamodel.test.TestCompiler printUsage
    } output
    list $output
} {{
Usage: java metropolis.metamodel.Compiler
       [-h]  [-?]  [-help]
       [-metamodel [-dumpast]]
       [-simulator]
       [-javasim]
       [-cpp]
       [-systemc [-top <netinitiator>] [-java <path>/java] [-javac <path>/javac] [-w] [-mmdebug]]
       [-elaborator <netinitiator>]
       [-promela]
       [-cfa]
       [-classpath <classpath>]
       source_files

    Use 'java metropolis.metamodel.Compiler -h' for more help. 

}}

######################################################################
####
#
test Compiler-3.1 {null second arg} {
    set args [java::new {String[]} 2 \
	[list "-metamodel"]]	
    catch {java::call metropolis.metamodel.test.TestCompiler main $args} \
	output
    list $output
} {{java.lang.RuntimeException: Compiler: Internal error: argument #1 is null.}}


######################################################################
####
#
# test Compiler-3.2 {no args} {
#     set args [java::new {String[]} 0 {}]
#     jdkCaptureErr {
# 	java::call metropolis.metamodel.test.TestCompiler main $args
#     } output
#     list $output
# } {{
# Usage: java metropolis.metamodel.Compiler
#        [-h]  [-?]  [-help]
#        [-metamodel [-dumpast]]
#        [-simulator]
#        [-javasim]
#        [-cpp]
#        [-systemc [-top <netinitiator>] [-java <path>/java] [-javac <path>/javac] [-w] [-mmdebug]]
#        [-elaborator <netinitiator>]
#        [-promela]
#        [-cfa]
#        [-classpath <classpath>]
#        source_files
# 
#     Use 'java metropolis.metamodel.Compiler -h' for more help. 
# 
# }}
# 
# ######################################################################
# ####
# #
# test Compiler-3.3 {Only arg is -} {
#     set args [java::new {String[]} 1 \
# 	[list "-"]]
#     jdkCaptureErr {
# 	java::call metropolis.metamodel.test.TestCompiler printUsage
#     } output
#     list $output
# } {{
# 
